package com.agiledeveloper.ticktacktoe;

import junit.framework.TestCase;

/**
 * User: Venkat Subramaniam
 * Date: Jul 11, 2004
 * Time: 8:56:55 AM
 */
public class TickTackToeBoardTest extends TestCase
{
    private TickTackToeBoard board;

    protected void setUp() throws Exception
    {
        super.setUp();
        board = new TickTackToeBoard();
    }

    public void testNewGameNotOver()
    {
        assertFalse(board.isGameOver());
    }

    public void testSetFirstPlayer()
    {
        board.firstPlayerIsX(true);
        assertTrue("Player should have been X since first player is set to X", board.isNextPlayerPegX());
    }

    public void testChangeFirstPlayer()
    {
        board.firstPlayerIsX(true);
        assertTrue(board.isNextPlayerPegX());
        board.firstPlayerIsX(false);
        assertFalse(board.isNextPlayerPegX());
    }

    public void testCheckNextPlayerAtBeginning()
    {
        try
        {
            board.isNextPlayerPegX();
            fail("Should have thrown a TickTackToeBoardException");
        }
        catch (TickTackToeBoardException e)
        {
            // succeeded
        }
    }

    public void testPlaceFirstPeg()
    {
        board.firstPlayerIsX(true);
        board.placePeg(1, 2);
        assertTrue(board.isPegXAt(1, 2));
    }

    public void testPlacePegOutOfRange()
    {
        try
        {
            board.firstPlayerIsX(false);
            board.placePeg(1, 3);
            fail("Expected TickTackToeBoardException for peg out of range");
        }
        catch (TickTackToeBoardException e)
        {
            //Success
        }
    }

    public void testGetOutOfRange()
    {
        try
        {
            board.isPegXAt(1, 3);
            fail("Expected TickTackToeBoardException for peg out of range");
        }
        catch (TickTackToeBoardException e)
        {
            //Success
        }
    }

    public void testPlacePegAtOccupiedPosition()
    {
        try
        {
            board.firstPlayerIsX(false);
            board.placePeg(1, 2);
            board.placePeg(1, 2);
            fail("Expected TickTackToeBoardException for peg out of range");
        }
        catch (TickTackToeBoardException e)
        {
            //Success
        }
    }

    public void testPlacePegWithoutSettingFirstPlayer()
    {
        try
        {
            board.placePeg(1, 2);
            fail("Expected TickTackToeBoardException for peg out of range");
        }
        catch (TickTackToeBoardException e)
        {
            //Success
        }
    }

    public void testSetFirstPlayerAfterGameBegins()
    {
        try
        {
            board.firstPlayerIsX(true);
            board.placePeg(1, 2);
            board.firstPlayerIsX(true);
            fail("Expected TickTackToeBoardException");
        }
        catch (TickTackToeBoardException e)
        {
            //Success
        }
    }

    public void testWinGameRowMatch()
    {
        board.firstPlayerIsX(true);
        board.placePeg(0, 0); // X
        board.placePeg(2, 2); // O
        board.placePeg(0, 1); // X
        board.placePeg(1, 2); // O
        board.placePeg(0, 2); // X
        assertTrue(board.isGameOver());
    }

    public void testWinGameColumnMatch()
    {
        board.firstPlayerIsX(false);
        board.placePeg(0, 1); // O
        board.placePeg(0, 0); // X
        board.placePeg(2, 1); // O
        board.placePeg(2, 2); // X
        board.placePeg(1, 1); // O
        assertTrue(board.isGameOver());
    }

    public void testWinGameDiagonalMatch()
    {
        board.firstPlayerIsX(false);
        board.placePeg(0, 0); // O
        board.placePeg(0, 2); // X
        board.placePeg(1, 1); // O
        board.placePeg(2, 1); // X
        board.placePeg(2, 2); // O
        assertTrue(board.isGameOver());
    }

    public void testPlacePegAfterGameOver()
    {
        try
        {
            testWinGameRowMatch();
            board.placePeg(0, 1);
            fail("Expected TickTackToeException");
            // Note, this is nuts. Do not use one test in another.
            // It has way too many side effects. Noted.
        }
        catch (TickTackToeBoardException e)
        {
            //Success
        }
    }

//    public static void main(String[] args)
//    {
//        junit.swingui.TestRunner.run(TickTackToeBoardTest.class);
//    }
}